@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2023 Laurent Gomila (laurent@sfml-dev.org)
+// Copyright (C) 2007-2024 Laurent Gomila (laurent@sfml-dev.org)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -22,42 +22,58 @@
 //
 ////////////////////////////////////////////////////////////
 
-#ifndef SFML_OUTPUTSOUNDFILE_HPP
-#define SFML_OUTPUTSOUNDFILE_HPP
+#pragma once
 
 ////////////////////////////////////////////////////////////
 // Headers
 ////////////////////////////////////////////////////////////
 #include <SFML/Audio/Export.hpp>
-#include <SFML/System/NonCopyable.hpp>
-#include <string>
+
+#include <SFML/Audio/SoundChannel.hpp>
+#include <SFML/Audio/SoundFileWriter.hpp>
+
+#include <filesystem>
+#include <memory>
+#include <vector>
+
+#include <cstdint>
 
 
 namespace sf
 {
-class SoundFileWriter;
-
 ////////////////////////////////////////////////////////////
 /// \brief Provide write access to sound files
 ///
 ////////////////////////////////////////////////////////////
-class SFML_AUDIO_API OutputSoundFile : NonCopyable
+class SFML_AUDIO_API OutputSoundFile
 {
 public:
-
     ////////////////////////////////////////////////////////////
     /// \brief Default constructor
     ///
+    /// Construct an output sound file that is not associated
+    /// with a file to write.
+    ///
     ////////////////////////////////////////////////////////////
-    OutputSoundFile();
+    OutputSoundFile() = default;
 
     ////////////////////////////////////////////////////////////
-    /// \brief Destructor
+    /// \brief Construct the sound file from the disk for writing
     ///
-    /// Closes the file if it was still open.
+    /// The supported audio formats are: WAV, OGG/Vorbis, FLAC.
+    ///
+    /// \param filename     Path of the sound file to write
+    /// \param sampleRate   Sample rate of the sound
+    /// \param channelCount Number of channels in the sound
+    /// \param channelMap   Map of position in sample frame to sound channel
+    ///
+    /// \throws sf::Exception if the file could not be opened successfully
     ///
     ////////////////////////////////////////////////////////////
-    ~OutputSoundFile();
+    OutputSoundFile(const std::filesystem::path&     filename,
+                    unsigned int                     sampleRate,
+                    unsigned int                     channelCount,
+                    const std::vector<SoundChannel>& channelMap);
 
     ////////////////////////////////////////////////////////////
     /// \brief Open the sound file from the disk for writing
@@ -67,11 +83,15 @@ public:
     /// \param filename     Path of the sound file to write
     /// \param sampleRate   Sample rate of the sound
     /// \param channelCount Number of channels in the sound
+    /// \param channelMap   Map of position in sample frame to sound channel
     ///
-    /// \return True if the file was successfully opened
+    /// \return `true` if the file was successfully opened
     ///
     ////////////////////////////////////////////////////////////
-    bool openFromFile(const std::string& filename, unsigned int sampleRate, unsigned int channelCount);
+    [[nodiscard]] bool openFromFile(const std::filesystem::path&     filename,
+                                    unsigned int                     sampleRate,
+                                    unsigned int                     channelCount,
+                                    const std::vector<SoundChannel>& channelMap);
 
     ////////////////////////////////////////////////////////////
     /// \brief Write audio samples to the file
@@ -80,7 +100,7 @@ public:
     /// \param count       Number of samples to write
     ///
     ////////////////////////////////////////////////////////////
-    void write(const Int16* samples, Uint64 count);
+    void write(const std::int16_t* samples, std::uint64_t count);
 
     ////////////////////////////////////////////////////////////
     /// \brief Close the current file
@@ -89,45 +109,39 @@ public:
     void close();
 
 private:
-
     ////////////////////////////////////////////////////////////
     // Member data
     ////////////////////////////////////////////////////////////
-    SoundFileWriter* m_writer; //!< Writer that handles I/O on the file's format
+    std::unique_ptr<SoundFileWriter> m_writer; //!< Writer that handles I/O on the file's format
 };
 
 } // namespace sf
 
 
-#endif // SFML_OUTPUTSOUNDFILE_HPP
-
-
 ////////////////////////////////////////////////////////////
 /// \class sf::OutputSoundFile
 /// \ingroup audio
 ///
 /// This class encodes audio samples to a sound file. It is
-/// used internally by higher-level classes such as sf::SoundBuffer,
+/// used internally by higher-level classes such as `sf::SoundBuffer`,
 /// but can also be useful if you want to create audio files from
 /// custom data sources, like generated audio samples.
 ///
 /// Usage example:
 /// \code
 /// // Create a sound file, ogg/vorbis format, 44100 Hz, stereo
-/// sf::OutputSoundFile file;
-/// if (!file.openFromFile("music.ogg", 44100, 2))
-///     /* error */;
+/// sf::OutputSoundFile file("music.ogg", 44100, 2, {sf::SoundChannel::FrontLeft, sf::SoundChannel::FrontRight});
 ///
 /// while (...)
 /// {
 ///     // Read or generate audio samples from your custom source
-///     std::vector<sf::Int16> samples = ...;
+///     std::vector<std::int16_t> samples = ...;
 ///
 ///     // Write them to the file
 ///     file.write(samples.data(), samples.size());
 /// }
 /// \endcode
 ///
-/// \see sf::SoundFileWriter, sf::InputSoundFile
+/// \see `sf::SoundFileWriter`, `sf::InputSoundFile`
 ///
 ////////////////////////////////////////////////////////////
