@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2016 Laurent Gomila (laurent@sfml-dev.org)
+// Copyright (C) 2007-2024 Laurent Gomila (laurent@sfml-dev.org)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -22,79 +22,76 @@
 //
 ////////////////////////////////////////////////////////////
 
-#ifndef SFML_WINDOWHANDLE_HPP
-#define SFML_WINDOWHANDLE_HPP
+#pragma once
 
 ////////////////////////////////////////////////////////////
 // Headers
 ////////////////////////////////////////////////////////////
 #include <SFML/Config.hpp>
 
-// Windows' HWND is a typedef on struct HWND__*
+// Windows' HWND is a type alias for struct HWND__*
 #if defined(SFML_SYSTEM_WINDOWS)
-    struct HWND__;
+struct HWND__; // NOLINT(bugprone-reserved-identifier)
 #endif
 
 namespace sf
 {
 #if defined(SFML_SYSTEM_WINDOWS)
 
-    // Window handle is HWND (HWND__*) on Windows
-    typedef HWND__* WindowHandle;
+// Window handle is HWND (HWND__*) on Windows
+using WindowHandle = HWND__*;
 
-#elif defined(SFML_SYSTEM_LINUX) || defined(SFML_SYSTEM_FREEBSD)
+#elif defined(SFML_SYSTEM_LINUX) || defined(SFML_SYSTEM_FREEBSD) || defined(SFML_SYSTEM_OPENBSD) || \
+    defined(SFML_SYSTEM_NETBSD)
 
-    // Window handle is Window (unsigned long) on Unix - X11
-    typedef unsigned long WindowHandle;
+// Window handle is Window (unsigned long) on Unix - X11
+using WindowHandle = unsigned long;
 
 #elif defined(SFML_SYSTEM_MACOS)
 
-    // Window handle is NSWindow or NSView (void*) on Mac OS X - Cocoa
-    typedef void* WindowHandle;
+// Window handle is NSWindow or NSView (void*) on macOS - Cocoa
+using WindowHandle = void*;
 
 #elif defined(SFML_SYSTEM_IOS)
 
-    // Window handle is UIWindow (void*) on iOS - UIKit
-    typedef void* WindowHandle;
+// Window handle is UIWindow (void*) on iOS - UIKit
+using WindowHandle = void*;
 
 #elif defined(SFML_SYSTEM_ANDROID)
 
-    // Window handle is ANativeWindow* (void*) on Android
-    typedef void* WindowHandle;
+// Window handle is ANativeWindow* (void*) on Android
+using WindowHandle = void*;
 
 #elif defined(SFML_DOXYGEN)
 
-    // Define typedef symbol so that Doxygen can attach some documentation to it
-    typedef "platform–specific" WindowHandle;
+// Define type alias symbol so that Doxygen can attach some documentation to it
+using WindowHandle = "platform-specific";
 
 #endif
 
 } // namespace sf
 
 
-#endif // SFML_WINDOWHANDLE_HPP
-
 ////////////////////////////////////////////////////////////
 /// \typedef sf::WindowHandle
 /// \ingroup window
 ///
-/// Define a low-level window handle type, specific to
-/// each platform.
+/// Low-level window handle type, specific to each platform.
 ///
 /// Platform        | Type
 /// ----------------|------------------------------------------------------------
 /// Windows         | \p HWND
 /// Linux/FreeBSD   | \p %Window
-/// Mac OS X        | either \p NSWindow* or \p NSView*, disguised as \p void*
+/// macOS           | either \p NSWindow* or \p NSView*, disguised as \p void*
 /// iOS             | \p UIWindow*
 /// Android         | \p ANativeWindow*
 ///
-/// \par Mac OS X Specification
+/// \par macOS Specification
 ///
-/// On Mac OS X, a sf::Window can be created either from an
+/// On macOS, a `sf::Window` can be created either from an
 /// existing \p NSWindow* or an \p NSView*. When the window
 /// is created from a window, SFML will use its content view
-/// as the OpenGL area. sf::Window::getSystemHandle() will
+/// as the OpenGL area. `sf::Window::getNativeHandle()` will
 /// return the handle that was used to create the window,
 /// which is a \p NSWindow* by default.
 ///
