@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2023 Laurent Gomila (laurent@sfml-dev.org)
+// Copyright (C) 2007-2024 Laurent Gomila (laurent@sfml-dev.org)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -25,40 +25,31 @@
 ////////////////////////////////////////////////////////////
 // Headers
 ////////////////////////////////////////////////////////////
-#include <SFML/Graphics/Shape.hpp>
 #include <SFML/Graphics/RenderTarget.hpp>
+#include <SFML/Graphics/Shape.hpp>
 #include <SFML/Graphics/Texture.hpp>
-#include <cmath>
 
+#include <algorithm>
+
+#include <cassert>
+#include <cstddef>
 
 namespace
 {
-    // Compute the normal of a segment
-    sf::Vector2f computeNormal(const sf::Vector2f& p1, const sf::Vector2f& p2)
-    {
-        sf::Vector2f normal(p1.y - p2.y, p2.x - p1.x);
-        float length = std::sqrt(normal.x * normal.x + normal.y * normal.y);
-        if (length != 0.f)
-            normal /= length;
-        return normal;
-    }
-
-    // Compute the dot product of two vectors
-    float dotProduct(const sf::Vector2f& p1, const sf::Vector2f& p2)
-    {
-        return p1.x * p2.x + p1.y * p2.y;
-    }
+// Compute the normal of a segment
+sf::Vector2f computeNormal(sf::Vector2f p1, sf::Vector2f p2)
+{
+    sf::Vector2f normal = (p2 - p1).perpendicular();
+    const float  length = normal.length();
+    if (length != 0.f)
+        normal /= length;
+    return normal;
 }
+} // namespace
 
 
 namespace sf
 {
-////////////////////////////////////////////////////////////
-Shape::~Shape()
-{
-}
-
-
 ////////////////////////////////////////////////////////////
 void Shape::setTexture(const Texture* texture, bool resetRect)
 {
@@ -66,7 +57,7 @@ void Shape::setTexture(const Texture* texture, bool resetRect)
     {
         // Recompute the texture area if requested, or if there was no texture & rect before
         if (resetRect || (!m_texture && (m_textureRect == IntRect())))
-            setTextureRect(IntRect(0, 0, static_cast<int>(texture->getSize().x), static_cast<int>(texture->getSize().y)));
+            setTextureRect(IntRect({0, 0}, Vector2i(texture->getSize())));
     }
 
     // Assign the new texture
@@ -97,7 +88,7 @@ const IntRect& Shape::getTextureRect() const
 
 
 ////////////////////////////////////////////////////////////
-void Shape::setFillColor(const Color& color)
+void Shape::setFillColor(Color color)
 {
     m_fillColor = color;
     updateFillColors();
@@ -105,14 +96,14 @@ void Shape::setFillColor(const Color& color)
 
 
 ////////////////////////////////////////////////////////////
-const Color& Shape::getFillColor() const
+Color Shape::getFillColor() const
 {
     return m_fillColor;
 }
 
 
 ////////////////////////////////////////////////////////////
-void Shape::setOutlineColor(const Color& color)
+void Shape::setOutlineColor(Color color)
 {
     m_outlineColor = color;
     updateOutlineColors();
@@ -120,7 +111,7 @@ void Shape::setOutlineColor(const Color& color)
 
 
 ////////////////////////////////////////////////////////////
-const Color& Shape::getOutlineColor() const
+Color Shape::getOutlineColor() const
 {
     return m_outlineColor;
 }
@@ -141,6 +132,56 @@ float Shape::getOutlineThickness() const
 }
 
 
+////////////////////////////////////////////////////////////
+Vector2f Shape::getGeometricCenter() const
+{
+    const auto count = getPointCount();
+
+    switch (count)
+    {
+        case 0:
+            assert(false && "Cannot calculate geometric center of shape with no points");
+            return Vector2f{};
+        case 1:
+            return getPoint(0);
+        case 2:
+            return (getPoint(0) + getPoint(1)) / 2.f;
+        default: // more than two points
+            Vector2f centroid;
+            float    twiceArea = 0;
+
+            auto previousPoint = getPoint(count - 1);
+            for (std::size_t i = 0; i < count; ++i)
+            {
+                const auto  currentPoint = getPoint(i);
+                const float product      = previousPoint.cross(currentPoint);
+                twiceArea += product;
+                centroid += (currentPoint + previousPoint) * product;
+
+                previousPoint = currentPoint;
+            }
+
+            if (twiceArea != 0.f)
+            {
+                return centroid / 3.f / twiceArea;
+            }
+
+            // Fallback for no area - find the center of the bounding box
+            auto minPoint = getPoint(0);
+            auto maxPoint = minPoint;
+            for (std::size_t i = 1; i < count; ++i)
+            {
+                const auto currentPoint = getPoint(i);
+                minPoint.x              = std::min(minPoint.x, currentPoint.x);
+                maxPoint.x              = std::max(maxPoint.x, currentPoint.x);
+                minPoint.y              = std::min(minPoint.y, currentPoint.y);
+                maxPoint.y              = std::max(maxPoint.y, currentPoint.y);
+            }
+            return (maxPoint + minPoint) / 2.f;
+    }
+}
+
+
 ////////////////////////////////////////////////////////////
 FloatRect Shape::getLocalBounds() const
 {
@@ -155,30 +196,15 @@ FloatRect Shape::getGlobalBounds() const
 }
 
 
-////////////////////////////////////////////////////////////
-Shape::Shape() :
-m_texture         (NULL),
-m_textureRect     (),
-m_fillColor       (255, 255, 255),
-m_outlineColor    (255, 255, 255),
-m_outlineThickness(0),
-m_vertices        (TriangleFan),
-m_outlineVertices (TriangleStrip),
-m_insideBounds    (),
-m_bounds          ()
-{
-}
-
-
 ////////////////////////////////////////////////////////////
 void Shape::update()
 {
     // Get the total number of points of the shape
-    std::size_t count = getPointCount();
+    const std::size_t count = getPointCount();
     if (count < 3)
     {
-        m_vertices.resize(0);
-        m_outlineVertices.resize(0);
+        m_vertices.clear();
+        m_outlineVertices.clear();
         return;
     }
 
@@ -190,12 +216,11 @@ void Shape::update()
     m_vertices[count + 1].position = m_vertices[1].position;
 
     // Update the bounding rectangle
-    m_vertices[0] = m_vertices[1]; // so that the result of getBounds() is correct
+    m_vertices[0]  = m_vertices[1]; // so that the result of getBounds() is correct
     m_insideBounds = m_vertices.getBounds();
 
     // Compute the center and make it the first vertex
-    m_vertices[0].position.x = m_insideBounds.left + m_insideBounds.width / 2;
-    m_vertices[0].position.y = m_insideBounds.top + m_insideBounds.height / 2;
+    m_vertices[0].position = m_insideBounds.getCenter();
 
     // Color
     updateFillColors();
@@ -212,6 +237,7 @@ void Shape::update()
 void Shape::draw(RenderTarget& target, RenderStates states) const
 {
     states.transform *= getTransform();
+    states.coordinateType = CoordinateType::Pixels;
 
     // Render the inside
     states.texture = m_texture;
@@ -220,7 +246,7 @@ void Shape::draw(RenderTarget& target, RenderStates states) const
     // Render the outline
     if (m_outlineThickness != 0)
     {
-        states.texture = NULL;
+        states.texture = nullptr;
         target.draw(m_outlineVertices, states);
     }
 }
@@ -237,14 +263,16 @@ void Shape::updateFillColors()
 ////////////////////////////////////////////////////////////
 void Shape::updateTexCoords()
 {
-    FloatRect convertedTextureRect = FloatRect(m_textureRect);
+    const FloatRect convertedTextureRect(m_textureRect);
+
+    // Make sure not to divide by zero when the points are aligned on a vertical or horizontal line
+    const Vector2f safeInsideSize(m_insideBounds.size.x > 0 ? m_insideBounds.size.x : 1.f,
+                                  m_insideBounds.size.y > 0 ? m_insideBounds.size.y : 1.f);
 
     for (std::size_t i = 0; i < m_vertices.getVertexCount(); ++i)
     {
-        float xratio = m_insideBounds.width > 0 ? (m_vertices[i].position.x - m_insideBounds.left) / m_insideBounds.width : 0;
-        float yratio = m_insideBounds.height > 0 ? (m_vertices[i].position.y - m_insideBounds.top) / m_insideBounds.height : 0;
-        m_vertices[i].texCoords.x = convertedTextureRect.left + convertedTextureRect.width * xratio;
-        m_vertices[i].texCoords.y = convertedTextureRect.top + convertedTextureRect.height * yratio;
+        const Vector2f ratio    = (m_vertices[i].position - m_insideBounds.position).componentWiseDiv(safeInsideSize);
+        m_vertices[i].texCoords = convertedTextureRect.position + convertedTextureRect.size.componentWiseMul(ratio);
     }
 }
 
@@ -260,17 +288,17 @@ void Shape::updateOutline()
         return;
     }
 
-    std::size_t count = m_vertices.getVertexCount() - 2;
+    const std::size_t count = m_vertices.getVertexCount() - 2;
     m_outlineVertices.resize((count + 1) * 2);
 
     for (std::size_t i = 0; i < count; ++i)
     {
-        std::size_t index = i + 1;
+        const std::size_t index = i + 1;
 
         // Get the two segments shared by the current point
-        Vector2f p0 = (i == 0) ? m_vertices[count].position : m_vertices[index - 1].position;
-        Vector2f p1 = m_vertices[index].position;
-        Vector2f p2 = m_vertices[index + 1].position;
+        const Vector2f p0 = (i == 0) ? m_vertices[count].position : m_vertices[index - 1].position;
+        const Vector2f p1 = m_vertices[index].position;
+        const Vector2f p2 = m_vertices[index + 1].position;
 
         // Compute their normal
         Vector2f n1 = computeNormal(p0, p1);
@@ -278,14 +306,14 @@ void Shape::updateOutline()
 
         // Make sure that the normals point towards the outside of the shape
         // (this depends on the order in which the points were defined)
-        if (dotProduct(n1, m_vertices[0].position - p1) > 0)
+        if (n1.dot(m_vertices[0].position - p1) > 0)
             n1 = -n1;
-        if (dotProduct(n2, m_vertices[0].position - p1) > 0)
+        if (n2.dot(m_vertices[0].position - p1) > 0)
             n2 = -n2;
 
         // Combine them to get the extrusion direction
-        float factor = 1.f + (n1.x * n2.x + n1.y * n2.y);
-        Vector2f normal = (n1 + n2) / factor;
+        const float    factor = 1.f + (n1.x * n2.x + n1.y * n2.y);
+        const Vector2f normal = (n1 + n2) / factor;
 
         // Update the outline points
         m_outlineVertices[i * 2 + 0].position = p1;
