@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2023 Laurent Gomila (laurent@sfml-dev.org)
+// Copyright (C) 2007-2024 Laurent Gomila (laurent@sfml-dev.org)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -25,24 +25,24 @@
 ////////////////////////////////////////////////////////////
 // Headers
 ////////////////////////////////////////////////////////////
+#include <SFML/System/Time.hpp>
 #include <SFML/System/Unix/SleepImpl.hpp>
-#include <errno.h>
-#include <time.h>
+
+#include <cerrno>
+#include <ctime>
 
 
-namespace sf
-{
-namespace priv
+namespace sf::priv
 {
 ////////////////////////////////////////////////////////////
 void sleepImpl(Time time)
 {
-    Int64 usecs = time.asMicroseconds();
+    const std::int64_t usecs = time.asMicroseconds();
 
     // Construct the time to wait
-    timespec ti;
+    timespec ti{};
+    ti.tv_sec  = static_cast<time_t>(usecs / 1000000);
     ti.tv_nsec = static_cast<long>((usecs % 1000000) * 1000);
-    ti.tv_sec = static_cast<time_t>(usecs / 1000000);
 
     // Wait...
     // If nanosleep returns -1, we check errno. If it is EINTR
@@ -54,6 +54,4 @@ void sleepImpl(Time time)
     }
 }
 
-} // namespace priv
-
-} // namespace sf
+} // namespace sf::priv
