@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2023 Laurent Gomila (laurent@sfml-dev.org)
+// Copyright (C) 2007-2024 Laurent Gomila (laurent@sfml-dev.org)
 //               2013-2013 David Demelier (demelier.david@gmail.com)
 //
 // This software is provided 'as-is', without any express or implied warranty.
@@ -27,15 +27,18 @@
 // Headers
 ////////////////////////////////////////////////////////////
 #include <SFML/Window/JoystickImpl.hpp>
-#include <sys/stat.h>
+
 #include <dirent.h>
 #include <fcntl.h>
-#include <unistd.h>
-#include <cstring>
-#include <map>
+#include <optional>
 #include <string>
+#include <sys/stat.h>
+#include <unistd.h>
+#include <unordered_map>
 #include <utility>
 
+#include <cstring>
+
 ////////////////////////////////////////////////////////////
 /// \brief This file implements FreeBSD driver joystick
 ///
@@ -49,135 +52,138 @@
 
 namespace
 {
-    std::map<unsigned int, std::string> plugged;
-    std::map<int, std::pair<int, int> > hatValueMap;
+std::unordered_map<unsigned int, std::string> plugged;
+std::unordered_map<int, std::pair<int, int>>  hatValueMap;
 
-    bool isJoystick(const char *name)
+bool isJoystick(const char* name)
+{
+    int file = ::open(name, O_RDONLY | O_NONBLOCK);
+
+    if (file < 0)
+        return false;
+
+    report_desc_t desc = hid_get_report_desc(file);
+
+    if (!desc)
     {
-        int file = ::open(name, O_RDONLY | O_NONBLOCK);
+        ::close(file);
+        return false;
+    }
 
-        if (file < 0)
-            return false;
+    int        id   = hid_get_report_id(file);
+    hid_data_t data = hid_start_parse(desc, 1 << hid_input, id);
 
-        report_desc_t desc = hid_get_report_desc(file);
-
-        if (!desc)
-        {
-            ::close(file);
-            return false;
-        }
-
-        int id = hid_get_report_id(file);
-        hid_data_t data = hid_start_parse(desc, 1 << hid_input, id);
-
-        if (!data)
-        {
-            hid_dispose_report_desc(desc);
-            ::close(file);
-            return false;
-        }
-
-        hid_item_t item;
-
-        // Assume it isn't
-        bool result = false;
-
-        while (hid_get_item(data, &item) > 0)
-        {
-            if ((item.kind == hid_collection) && (HID_PAGE(item.usage) == HUP_GENERIC_DESKTOP))
-            {
-                if ((HID_USAGE(item.usage) == HUG_JOYSTICK) || (HID_USAGE(item.usage) == HUG_GAME_PAD))
-                {
-                    result = true;
-                }
-            }
-        }
-
-        hid_end_parse(data);
+    if (!data)
+    {
         hid_dispose_report_desc(desc);
         ::close(file);
-
-        return result;
+        return false;
     }
 
-    void updatePluggedList()
+    hid_item_t item;
+
+    // Assume it isn't
+    bool result = false;
+
+    while (hid_get_item(data, &item) > 0)
     {
-        /*
-         * Devices /dev/uhid<x> are shared between joystick and any other
-         * human interface device. We need to iterate over all found devices
-         * and check if they are joysticks. The index of JoystickImpl::open
-         * does not match the /dev/uhid<index> device!
-         */
-        DIR* directory = opendir("/dev");
-
-        if (directory)
+        if ((item.kind == hid_collection) && (HID_PAGE(item.usage) == HUP_GENERIC_DESKTOP))
         {
-            int joystickCount = 0;
-            struct dirent* directoryEntry = readdir(directory);
-
-            while (directoryEntry && joystickCount < sf::Joystick::Count)
+            if ((HID_USAGE(item.usage) == HUG_JOYSTICK) || (HID_USAGE(item.usage) == HUG_GAME_PAD))
             {
-                if (!std::strncmp(directoryEntry->d_name, "uhid", 4))
-                {
-                    std::string name("/dev/");
-                    name += directoryEntry->d_name;
+                result = true;
+            }
+        }
+    }
 
-                    if (isJoystick(name.c_str()))
-                        plugged[static_cast<unsigned int>(joystickCount++)] = name;
-                }
+    hid_end_parse(data);
+    hid_dispose_report_desc(desc);
+    ::close(file);
 
-                directoryEntry = readdir(directory);
+    return result;
+}
+
+void updatePluggedList()
+{
+    /*
+     * Devices /dev/uhid<x> are shared between joystick and any other
+     * human interface device. We need to iterate over all found devices
+     * and check if they are joysticks. The index of JoystickImpl::open
+     * does not match the /dev/uhid<index> device!
+     */
+    if (DIR* directory = opendir("/dev"))
+    {
+        unsigned int   joystickCount  = 0;
+        struct dirent* directoryEntry = readdir(directory);
+
+        while (directoryEntry && joystickCount < sf::Joystick::Count)
+        {
+            if (!std::strncmp(directoryEntry->d_name, "uhid", 4))
+            {
+                std::string name("/dev/");
+                name += directoryEntry->d_name;
+
+                if (isJoystick(name.c_str()))
+                    plugged[static_cast<unsigned int>(joystickCount++)] = name;
             }
 
-            closedir(directory);
+            directoryEntry = readdir(directory);
         }
-    }
 
-    int usageToAxis(int usage)
-    {
-        switch (usage)
-        {
-            case HUG_X:  return sf::Joystick::X;
-            case HUG_Y:  return sf::Joystick::Y;
-            case HUG_Z:  return sf::Joystick::Z;
-            case HUG_RZ: return sf::Joystick::R;
-            case HUG_RX: return sf::Joystick::U;
-            case HUG_RY: return sf::Joystick::V;
-            default:     return -1;
-        }
-    }
-
-    void hatValueToSfml(int value, sf::priv::JoystickState& state)
-    {
-        state.axes[sf::Joystick::PovX] = static_cast<float>(hatValueMap[value].first);
-        state.axes[sf::Joystick::PovY] = static_cast<float>(hatValueMap[value].second);
+        closedir(directory);
     }
 }
 
-
-namespace sf
+std::optional<sf::Joystick::Axis> usageToAxis(int usage)
 {
-namespace priv
+    switch (usage)
+    {
+        case HUG_X:
+            return sf::Joystick::Axis::X;
+        case HUG_Y:
+            return sf::Joystick::Axis::Y;
+        case HUG_Z:
+            return sf::Joystick::Axis::Z;
+        case HUG_RZ:
+            return sf::Joystick::Axis::R;
+        case HUG_RX:
+            return sf::Joystick::Axis::U;
+        case HUG_RY:
+            return sf::Joystick::Axis::V;
+        default:
+            return std::nullopt;
+    }
+}
+
+void hatValueToSfml(int value, sf::priv::JoystickState& state)
+{
+    state.axes[sf::Joystick::Axis::PovX] = static_cast<float>(hatValueMap[value].first);
+    state.axes[sf::Joystick::Axis::PovY] = static_cast<float>(hatValueMap[value].second);
+}
+} // namespace
+
+
+namespace sf::priv
 {
 ////////////////////////////////////////////////////////////
 void JoystickImpl::initialize()
 {
-    hid_init(NULL);
+    hid_init(nullptr);
 
     // Do an initial scan
     updatePluggedList();
 
     // Map of hat values
-    hatValueMap[0] = std::make_pair(   0,    0); // center
+    hatValueMap[0] = std::make_pair(0, 0); // center
 
-    hatValueMap[1] = std::make_pair(   0, -100); // top
-    hatValueMap[3] = std::make_pair( 100,    0); // right
-    hatValueMap[5] = std::make_pair(   0,  100); // bottom
-    hatValueMap[7] = std::make_pair(-100,    0); // left
+    hatValueMap[1] = std::make_pair(0, -100); // top
+    hatValueMap[3] = std::make_pair(100, 0);  // right
+    hatValueMap[5] = std::make_pair(0, 100);  // bottom
+    hatValueMap[7] = std::make_pair(-100, 0); // left
 
-    hatValueMap[2] = std::make_pair( 100, -100); // top-right
-    hatValueMap[4] = std::make_pair( 100,  100); // bottom-right
-    hatValueMap[6] = std::make_pair(-100,  100); // bottom-left
+    hatValueMap[2] = std::make_pair(100, -100);  // top-right
+    hatValueMap[4] = std::make_pair(100, 100);   // bottom-right
+    hatValueMap[6] = std::make_pair(-100, 100);  // bottom-left
     hatValueMap[8] = std::make_pair(-100, -100); // top-left
 }
 
@@ -220,7 +226,7 @@ bool JoystickImpl::open(unsigned int index)
 
             // Then allocate a buffer for data retrieval
             m_length = hid_report_size(m_desc, hid_input, m_id);
-            m_buffer.resize(static_cast<size_t>(m_length));
+            m_buffer.resize(static_cast<std::size_t>(m_length));
 
             m_state.connected = true;
 
@@ -244,7 +250,7 @@ void JoystickImpl::close()
 JoystickCaps JoystickImpl::getCapabilities() const
 {
     JoystickCaps caps;
-    hid_item_t item;
+    hid_item_t   item;
 
     hid_data_t data = hid_start_parse(m_desc, 1 << hid_input, m_id);
 
@@ -256,21 +262,19 @@ JoystickCaps JoystickImpl::getCapabilities() const
 
             if (usage == HUP_BUTTON)
             {
-                caps.buttonCount++;
+                ++caps.buttonCount;
                 break;
             }
             else if (usage == HUP_GENERIC_DESKTOP)
             {
-                int axis = usageToAxis(usage);
-
                 if (usage == HUG_HAT_SWITCH)
                 {
-                    caps.axes[Joystick::PovX] = true;
-                    caps.axes[Joystick::PovY] = true;
+                    caps.axes[Joystick::Axis::PovX] = true;
+                    caps.axes[Joystick::Axis::PovY] = true;
                 }
-                else if (axis != -1)
+                else if (const std::optional<Joystick::Axis> axis = usageToAxis(usage))
                 {
-                    caps.axes[axis] = true;
+                    caps.axes[*axis] = true;
                 }
             }
         }
@@ -292,7 +296,7 @@ Joystick::Identification JoystickImpl::getIdentification() const
 ////////////////////////////////////////////////////////////
 JoystickState JoystickImpl::JoystickImpl::update()
 {
-    while (read(m_file, &m_buffer[0], static_cast<size_t>(m_length)) == m_length)
+    while (read(m_file, m_buffer.data(), static_cast<std::size_t>(m_length)) == m_length)
     {
         hid_data_t data = hid_start_parse(m_desc, 1 << hid_input, m_id);
 
@@ -300,7 +304,7 @@ JoystickState JoystickImpl::JoystickImpl::update()
         if (!data)
             continue;
 
-        int buttonIndex = 0;
+        int        buttonIndex = 0;
         hid_item_t item;
 
         while (hid_get_item(data, &item))
@@ -311,24 +315,23 @@ JoystickState JoystickImpl::JoystickImpl::update()
 
                 if (usage == HUP_BUTTON)
                 {
-                    m_state.buttons[buttonIndex++] = hid_get_data(&m_buffer[0], &item);
+                    m_state.buttons[buttonIndex++] = hid_get_data(m_buffer.data(), &item);
                 }
                 else if (usage == HUP_GENERIC_DESKTOP)
                 {
-                    int value = hid_get_data(&m_buffer[0], &item);
-                    int axis = usageToAxis(usage);
+                    int value = hid_get_data(m_buffer.data(), &item);
 
                     if (usage == HUG_HAT_SWITCH)
                     {
                         hatValueToSfml(value, m_state);
                     }
-                    else if (axis != -1)
+                    else if (const std::optional<Joystick::Axis> axis = usageToAxis(usage))
                     {
                         int minimum = item.logical_minimum;
                         int maximum = item.logical_maximum;
 
-                        value = (value - minimum) * 200 / (maximum - minimum) - 100;
-                        m_state.axes[axis] = static_cast<float>(value);
+                        value               = (value - minimum) * 200 / (maximum - minimum) - 100;
+                        m_state.axes[*axis] = static_cast<float>(value);
                     }
                 }
             }
@@ -340,6 +343,4 @@ JoystickState JoystickImpl::JoystickImpl::update()
     return m_state;
 }
 
-} // namespace priv
-
-} // namespace sf
+} // namespace sf::priv
