@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2018 Laurent Gomila (laurent@sfml-dev.org)
+// Copyright (C) 2007-2024 Laurent Gomila (laurent@sfml-dev.org)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -22,24 +22,23 @@
 //
 ////////////////////////////////////////////////////////////
 
-#ifndef SFML_WINDOWIMPLX11_HPP
-#define SFML_WINDOWIMPLX11_HPP
+#pragma once
 
 ////////////////////////////////////////////////////////////
 // Headers
 ////////////////////////////////////////////////////////////
 #include <SFML/Window/Event.hpp>
+#include <SFML/Window/WindowEnums.hpp> // Prevent conflict with macro None from Xlib
 #include <SFML/Window/WindowImpl.hpp>
-#include <SFML/System/String.hpp>
-#include <SFML/Window/WindowStyle.hpp> // Prevent conflict with macro None from Xlib
+
 #include <X11/Xlib.h>
-#include <deque>
 #include <X11/extensions/Xrandr.h>
 
+#include <deque>
+#include <memory>
 
-namespace sf
-{
-namespace priv
+
+namespace sf::priv
 {
 ////////////////////////////////////////////////////////////
 /// \brief Linux (X11) implementation of WindowImpl
@@ -48,31 +47,31 @@ namespace priv
 class WindowImplX11 : public WindowImpl
 {
 public:
-
     ////////////////////////////////////////////////////////////
     /// \brief Construct the window implementation from an existing control
     ///
     /// \param handle Platform-specific handle of the control
     ///
     ////////////////////////////////////////////////////////////
-    WindowImplX11(WindowHandle handle);
+    explicit WindowImplX11(WindowHandle handle);
 
     ////////////////////////////////////////////////////////////
     /// \brief Create the window implementation
     ///
     /// \param mode  Video mode to use
     /// \param title Title of the window
-    /// \param style Window style (resizable, fixed, or fullscren)
+    /// \param style Window style
+    /// \param state Window state
     /// \param settings Additional settings for the underlying OpenGL context
     ///
     ////////////////////////////////////////////////////////////
-    WindowImplX11(VideoMode mode, const String& title, unsigned long style, const ContextSettings& settings);
+    WindowImplX11(VideoMode mode, const String& title, std::uint32_t style, State state, const ContextSettings& settings);
 
     ////////////////////////////////////////////////////////////
     /// \brief Destructor
     ///
     ////////////////////////////////////////////////////////////
-    ~WindowImplX11();
+    ~WindowImplX11() override;
 
     ////////////////////////////////////////////////////////////
     /// \brief Get the OS-specific handle of the window
@@ -80,7 +79,7 @@ public:
     /// \return Handle of the window
     ///
     ////////////////////////////////////////////////////////////
-    virtual WindowHandle getSystemHandle() const;
+    [[nodiscard]] WindowHandle getNativeHandle() const override;
 
     ////////////////////////////////////////////////////////////
     /// \brief Get the position of the window
@@ -88,7 +87,7 @@ public:
     /// \return Position of the window, in pixels
     ///
     ////////////////////////////////////////////////////////////
-    virtual Vector2i getPosition() const;
+    [[nodiscard]] Vector2i getPosition() const override;
 
     ////////////////////////////////////////////////////////////
     /// \brief Change the position of the window on screen
@@ -96,7 +95,7 @@ public:
     /// \param position New position of the window, in pixels
     ///
     ////////////////////////////////////////////////////////////
-    virtual void setPosition(const Vector2i& position);
+    void setPosition(Vector2i position) override;
 
     ////////////////////////////////////////////////////////////
     /// \brief Get the client size of the window
@@ -104,7 +103,7 @@ public:
     /// \return Size of the window, in pixels
     ///
     ////////////////////////////////////////////////////////////
-    virtual Vector2u getSize() const;
+    [[nodiscard]] Vector2u getSize() const override;
 
     ////////////////////////////////////////////////////////////
     /// \brief Change the size of the rendering region of the window
@@ -112,7 +111,27 @@ public:
     /// \param size New size, in pixels
     ///
     ////////////////////////////////////////////////////////////
-    virtual void setSize(const Vector2u& size);
+    void setSize(Vector2u size) override;
+
+    ////////////////////////////////////////////////////////////
+    /// \brief Set the minimum window rendering region size
+    ///
+    /// Pass `std::nullopt` to unset the minimum size
+    ///
+    /// \param minimumSize New minimum size, in pixels
+    ///
+    ////////////////////////////////////////////////////////////
+    void setMinimumSize(const std::optional<Vector2u>& minimumSize) override;
+
+    ////////////////////////////////////////////////////////////
+    /// \brief Set the maximum window rendering region size
+    ///
+    /// Pass `std::nullopt` to unset the maximum size
+    ///
+    /// \param maximumSize New maximum size, in pixels
+    ///
+    ////////////////////////////////////////////////////////////
+    void setMaximumSize(const std::optional<Vector2u>& maximumSize) override;
 
     ////////////////////////////////////////////////////////////
     /// \brief Change the title of the window
@@ -120,41 +139,40 @@ public:
     /// \param title New title
     ///
     ////////////////////////////////////////////////////////////
-    virtual void setTitle(const String& title);
+    void setTitle(const String& title) override;
 
     ////////////////////////////////////////////////////////////
     /// \brief Change the window's icon
     ///
-    /// \param width  Icon's width, in pixels
-    /// \param height Icon's height, in pixels
+    /// \param size   Icon's width and height, in pixels
     /// \param pixels Pointer to the pixels in memory, format must be RGBA 32 bits
     ///
     ////////////////////////////////////////////////////////////
-    virtual void setIcon(unsigned int width, unsigned int height, const Uint8* pixels);
+    void setIcon(Vector2u size, const std::uint8_t* pixels) override;
 
     ////////////////////////////////////////////////////////////
     /// \brief Show or hide the window
     ///
-    /// \param visible True to show, false to hide
+    /// \param visible `true` to show, `false` to hide
     ///
     ////////////////////////////////////////////////////////////
-    virtual void setVisible(bool visible);
+    void setVisible(bool visible) override;
 
     ////////////////////////////////////////////////////////////
     /// \brief Show or hide the mouse cursor
     ///
-    /// \param visible True to show, false to hide
+    /// \param visible `true` to show, `false` to hide
     ///
     ////////////////////////////////////////////////////////////
-    virtual void setMouseCursorVisible(bool visible);
+    void setMouseCursorVisible(bool visible) override;
 
     ////////////////////////////////////////////////////////////
     /// \brief Grab or release the mouse cursor
     ///
-    /// \param grabbed True to enable, false to disable
+    /// \param grabbed `true` to enable, `false` to disable
     ///
     ////////////////////////////////////////////////////////////
-    virtual void setMouseCursorGrabbed(bool grabbed);
+    void setMouseCursorGrabbed(bool grabbed) override;
 
     ////////////////////////////////////////////////////////////
     /// \brief Set the displayed cursor to a native system cursor
@@ -162,41 +180,39 @@ public:
     /// \param cursor Native system cursor type to display
     ///
     ////////////////////////////////////////////////////////////
-    virtual void setMouseCursor(const CursorImpl& cursor);
+    void setMouseCursor(const CursorImpl& cursor) override;
 
     ////////////////////////////////////////////////////////////
     /// \brief Enable or disable automatic key-repeat
     ///
-    /// \param enabled True to enable, false to disable
+    /// \param enabled `true` to enable, `false` to disable
     ///
     ////////////////////////////////////////////////////////////
-    virtual void setKeyRepeatEnabled(bool enabled);
+    void setKeyRepeatEnabled(bool enabled) override;
 
     ////////////////////////////////////////////////////////////
     /// \brief Request the current window to be made the active
     ///        foreground window
     ///
     ////////////////////////////////////////////////////////////
-    virtual void requestFocus();
+    void requestFocus() override;
 
     ////////////////////////////////////////////////////////////
     /// \brief Check whether the window has the input focus
     ///
-    /// \return True if window has focus, false otherwise
+    /// \return `true` if window has focus, `false` otherwise
     ///
     ////////////////////////////////////////////////////////////
-    virtual bool hasFocus() const;
+    [[nodiscard]] bool hasFocus() const override;
 
 protected:
-
     ////////////////////////////////////////////////////////////
     /// \brief Process incoming events from the operating system
     ///
     ////////////////////////////////////////////////////////////
-    virtual void processEvents();
+    void processEvents() override;
 
 private:
-
     ////////////////////////////////////////////////////////////
     /// \brief Request the WM to make the current window active
     ///
@@ -260,34 +276,29 @@ private:
     ///
     /// \param windowEvent Event which has been received
     ///
-    /// \return True if the event was processed, false if it was discarded
+    /// \return `true` if the event was processed, `false` if it was discarded
     ///
     ////////////////////////////////////////////////////////////
     bool processEvent(XEvent& windowEvent);
 
     ////////////////////////////////////////////////////////////
-    /// \brief Check if a valid version of XRandR extension is present 
+    /// \brief Check if a valid version of XRandR extension is present
     ///
-    /// \param xRandRMajor XRandR major version
-    /// \param xRandRMinor XRandR minor version
-    ///
-    /// \return True if a valid XRandR version found, false otherwise
+    /// \return `true` if a valid XRandR version found, `false` otherwise
     ///
     ////////////////////////////////////////////////////////////
-    bool checkXRandR(int& xRandRMajor, int& xRandRMinor);
+    bool checkXRandR();
 
     ////////////////////////////////////////////////////////////
     /// \brief Get the RROutput of the primary monitor
     ///
     /// \param rootWindow the root window
     /// \param res screen resources
-    /// \param xRandRMajor XRandR major version
-    /// \param xRandRMinor XRandR minor version
     ///
     /// \return RROutput of the primary monitor
     ///
     ////////////////////////////////////////////////////////////
-    RROutput getOutputPrimary(::Window& rootWindow, XRRScreenResources* res, int xRandRMajor, int xRandRMinor);
+    RROutput getOutputPrimary(::Window& rootWindow, XRRScreenResources* res);
 
     ////////////////////////////////////////////////////////////
     /// \brief Get coordinates of the primary monitor
@@ -297,34 +308,34 @@ private:
     ////////////////////////////////////////////////////////////
     Vector2i getPrimaryMonitorPosition();
 
+    ////////////////////////////////////////////////////////////
+    /// \brief Set min/max window size
+    ///
+    ////////////////////////////////////////////////////////////
+    void setWindowSizeConstraints() const;
+
     ////////////////////////////////////////////////////////////
     // Member data
     ////////////////////////////////////////////////////////////
-    ::Window           m_window;         ///< X identifier defining our window
-    ::Display*         m_display;        ///< Pointer to the display
-    int                m_screen;         ///< Screen identifier
-    XIM                m_inputMethod;    ///< Input method linked to the X display
-    XIC                m_inputContext;   ///< Input context used to get unicode input in our window
-    std::deque<XEvent> m_events;         ///< Queue we use to store pending events for this window
-    bool               m_isExternal;     ///< Tell whether the window has been created externally or by SFML
-    int                m_oldVideoMode;   ///< Video mode in use before we switch to fullscreen
-    RRCrtc             m_oldRRCrtc;      ///< RRCrtc in use before we switch to fullscreen
-    ::Cursor           m_hiddenCursor;   ///< As X11 doesn't provide cursor hiding, we must create a transparent one
-    ::Cursor           m_lastCursor;     ///< Last cursor used -- this data is not owned by the window and is required to be always valid
-    bool               m_keyRepeat;      ///< Is the KeyRepeat feature enabled?
-    Vector2i           m_previousSize;   ///< Previous size of the window, to find if a ConfigureNotify event is a resize event (could be a move event only)
-    bool               m_useSizeHints;   ///< Is the size of the window fixed with size hints?
-    bool               m_fullscreen;     ///< Is the window in fullscreen?
-    bool               m_cursorGrabbed;  ///< Is the mouse cursor trapped?
-    bool               m_windowMapped;   ///< Has the window been mapped by the window manager?
-    Pixmap             m_iconPixmap;     ///< The current icon pixmap if in use
-    Pixmap             m_iconMaskPixmap; ///< The current icon mask pixmap if in use
-    ::Time             m_lastInputTime;  ///< Last time we received user input
+    ::Window                 m_window{};       ///< X identifier defining our window
+    std::shared_ptr<Display> m_display;        ///< Pointer to the display
+    int                      m_screen;         ///< Screen identifier
+    std::shared_ptr<_XIM>    m_inputMethod;    ///< Input method linked to the X display
+    XIC                      m_inputContext{}; ///< Input context used to get unicode input in our window
+    bool                     m_isExternal{};   ///< Tell whether the window has been created externally or by SFML
+    RRMode                   m_oldVideoMode{}; ///< Video mode in use before we switch to fullscreen
+    RRCrtc                   m_oldRRCrtc{};    ///< RRCrtc in use before we switch to fullscreen
+    ::Cursor m_hiddenCursor{}; ///< As X11 doesn't provide cursor hiding, we must create a transparent one
+    ::Cursor m_lastCursor{None}; ///< Last cursor used -- this data is not owned by the window and is required to be always valid
+    bool m_keyRepeat{true}; ///< Is the KeyRepeat feature enabled?
+    Vector2i m_previousSize{-1, -1}; ///< Previous size of the window, to find if a ConfigureNotify event is a resize event (could be a move event only)
+    bool   m_useSizeHints{};   ///< Is the size of the window fixed with size hints?
+    bool   m_fullscreen{};     ///< Is the window in fullscreen?
+    bool   m_cursorGrabbed{};  ///< Is the mouse cursor trapped?
+    bool   m_windowMapped{};   ///< Has the window been mapped by the window manager?
+    Pixmap m_iconPixmap{};     ///< The current icon pixmap if in use
+    Pixmap m_iconMaskPixmap{}; ///< The current icon mask pixmap if in use
+    ::Time m_lastInputTime{};  ///< Last time we received user input
 };
 
-} // namespace priv
-
-} // namespace sf
-
-
-#endif // SFML_WINDOWIMPLX11_HPP
+} // namespace sf::priv
