@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2023 Laurent Gomila (laurent@sfml-dev.org)
+// Copyright (C) 2007-2024 Laurent Gomila (laurent@sfml-dev.org)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -22,18 +22,17 @@
 //
 ////////////////////////////////////////////////////////////
 
-#ifndef SFML_VIDEOMODEIMPL_HPP
-#define SFML_VIDEOMODEIMPL_HPP
+#pragma once
 
 ////////////////////////////////////////////////////////////
 // Headers
 ////////////////////////////////////////////////////////////
 #include <SFML/Window/VideoMode.hpp>
 
+#include <vector>
 
-namespace sf
-{
-namespace priv
+
+namespace sf::priv
 {
 ////////////////////////////////////////////////////////////
 /// \brief OS-specific implementation of video modes functions
@@ -42,7 +41,6 @@ namespace priv
 class VideoModeImpl
 {
 public:
-
     ////////////////////////////////////////////////////////////
     /// \brief Get the list of all the supported fullscreen video modes
     ///
@@ -60,9 +58,4 @@ public:
     static VideoMode getDesktopMode();
 };
 
-} // namespace priv
-
-} // namespace sf
-
-
-#endif // SFML_VIDEOMODEIMPL_HPP
+} // namespace sf::priv
